@@ -34,12 +34,10 @@ def emulate_async(fn):
 
 
 class AwaitableCallback(object):
-    """A sync callback whose completion can be waited upon.
-    """
+    """A sync callback whose completion can be waited upon."""
 
     def __init__(self, return_arg_name=None):
-        """Creates an instance of an AwaitableCallback
-        """
+        """Creates an instance of an AwaitableCallback"""
 
         # LBYL because this mistake doesn't cause an exception until the callback
         # which is much later and very difficult to trace back to here.
@@ -70,9 +68,9 @@ class AwaitableCallback(object):
                 result = None
 
             if exception:
-                # Do not use exc_info parameter on logger.error.  This casuses pytest to save the traceback which saves stack frames which shows up as a leak
-                logger.error("Callback completed with error {}".format(exception))
-                logger.error(traceback.format_exception_only(type(exception), exception))
+                # Do not use exc_info parameter on logger.* calls.  This casuses pytest to save the traceback which saves stack frames which shows up as a leak
+                logger.info("Callback completed with error {}".format(exception))
+                logger.info(traceback.format_exception_only(type(exception), exception))
                 loop.call_soon_threadsafe(self.future.set_exception, exception)
             else:
                 logger.debug("Callback completed with result {}".format(result))
@@ -81,8 +79,7 @@ class AwaitableCallback(object):
         self.callback = wrapping_callback
 
     def __call__(self, *args, **kwargs):
-        """Calls the callback. Returns the result.
-        """
+        """Calls the callback. Returns the result."""
         return self.callback(*args, **kwargs)
 
     async def completion(self):
