@@ -20,6 +20,12 @@ class OperationCancelled(PipelineException):
     pass
 
 
+class OperationTimeout(PipelineException):
+    """Pipeline operation timed out"""
+
+    pass
+
+
 class OperationError(PipelineException):
     """Error while executing an Operation"""
 
@@ -32,12 +38,6 @@ class PipelineNotRunning(PipelineException):
     pass
 
 
-class PipelineTimeoutError(PipelineException):
-    """Pipeline operation timed out"""
-
-    pass
-
-
 class PipelineRuntimeError(PipelineException):
     """Error at runtime caused by incorrect pipeline configuration"""
 
