@@ -19,7 +19,7 @@ from azure.iot.device.common.auth import connection_string as cs
 from azure.iot.device.common.auth import sastoken as st
 from azure.iot.device.iothub import client_event
 from azure.iot.device import exceptions
-from azure.iot.device.common import auth
+from azure.iot.device.common import auth, handle_exceptions
 from . import edge_hsm
 
 logger = logging.getLogger(__name__)
@@ -141,11 +141,19 @@ class AbstractIoTHubClient(object):
         logger.info("Cleared all pending method requests due to disconnect")
 
     def _on_new_sastoken_required(self):
+        """Helper handler that is called upon the iothub pipeline needing new SAS token"""
         logger.info("New SasToken required from user")
         client_event_inbox = self._inbox_manager.get_client_event_inbox()
         event = client_event.ClientEvent(client_event.NEW_SASTOKEN_REQUIRED)
         client_event_inbox.put(event)
 
+    def _on_background_exception(self, e):
+        """Helper handler that is called upon an iothub pipeline background exception"""
+        handle_exceptions.handle_background_exception(e)
+        client_event_inbox = self._inbox_manager.get_client_event_inbox()
+        event = client_event.ClientEvent(client_event.BACKGROUND_EXCEPTION, e)
+        client_event_inbox.put(event)
+
     def _check_receive_mode_is_api(self):
         """Call this function first in EVERY receive API"""
         with self._client_lock:
@@ -438,13 +446,17 @@ class AbstractIoTHubClient(object):
     def on_new_sastoken_required(self, value):
         self._handler_manager.on_new_sastoken_required = value
 
-    # @property
-    # def on_background_exception(self):
-    #     return self._handler_manager.on_background_exception
+    @property
+    def on_background_exception(self):
+        """The handler function or coroutine will be called when a background exception occurs.
 
-    # @on_background_exception.setter
-    # def on_background_exception(self, value):
-    #     self._handler_manager.on_background_exception = value
+        The function or coroutine definition should take one positional argument (the exception
+        object)"""
+        return self._handler_manager.on_background_exception
+
+    @on_background_exception.setter
+    def on_background_exception(self, value):
+        self._handler_manager.on_background_exception = value
 
     @abc.abstractproperty
     def on_message_received(self):
