@@ -45,6 +45,7 @@ class MQTTPipeline(object):
         self.on_connected = None
         self.on_disconnected = None
         self.on_new_sastoken_required = None
+        self.on_background_exception = None
 
         self.on_c2d_message_received = None
         self.on_input_message_received = None
@@ -170,11 +171,20 @@ class MQTTPipeline(object):
             else:
                 logger.debug("IoTHub Pipeline requires new SASToken, but no handler was set")
 
+        def _on_background_exception(e):
+            if self.on_background_exception:
+                self.on_background_exception(e)
+            else:
+                logger.debug(
+                    "IoTHub Pipeline experienced background exception, but no handler was set"
+                )
+
         # Set internal event handlers
         self._pipeline.on_pipeline_event_handler = _on_pipeline_event
         self._pipeline.on_connected_handler = _on_connected
         self._pipeline.on_disconnected_handler = _on_disconnected
         self._pipeline.on_new_sastoken_required_handler = _on_new_sastoken_required
+        self._pipeline.on_background_exception_handler = _on_background_exception
 
         # Initialize the pipeline
         callback = EventedCallback()
@@ -226,7 +236,7 @@ class MQTTPipeline(object):
         """
         Connect to the service.
 
-        :param callback: callback which is called when the connection to the service is complete.
+        :param callback: callback which is called when the connection attempt is complete.
 
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.PipelineNotRunning` if the
             pipeline has previously been shut down
@@ -238,6 +248,7 @@ class MQTTPipeline(object):
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
         logger.debug("Starting ConnectOperation on the pipeline")
@@ -251,7 +262,9 @@ class MQTTPipeline(object):
         """
         Disconnect from the service.
 
-        :param callback: callback which is called when the connection to the service has been disconnected
+        Note that even if this fails for some reason, the client will be in a disconnected state.
+
+        :param callback: callback which is called when the disconnection is complete.
 
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.PipelineNotRunning` if the
             pipeline has previously been shut down
@@ -271,13 +284,13 @@ class MQTTPipeline(object):
 
     def reauthorize_connection(self, callback):
         """
-        Reauthorize connection to the service.
+        Reauthorize connection to the service by disconnecting and then reconnecting using
+        fresh credentials.
 
-        Technically, this function will return upon disconnection. The disconnection will then
-        immediately trigger a reconnect, but this function will not wait for that to return.
-        This is (unfortunately) necessary while supporting MQTT3.
+        This can be called regardless of connection state. If successful, the client will be
+        connected. If unsuccessful, the client will be disconnected.
 
-        :param callback: callback which is called when the connection to the service has been disconnected
+        :param callback: callback which is called when the reauthorization attempt is complete.
 
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.PipelineNotRunning` if the
             pipeline has previously been shut down
@@ -285,7 +298,11 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionFailedError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
         logger.debug("Starting ReauthorizeConnectionOperation on the pipeline")
@@ -302,7 +319,7 @@ class MQTTPipeline(object):
         Send a telemetry message to the service.
 
         :param message: message to send.
-        :param callback: callback which is called when the message publish has been acknowledged by the service.
+        :param callback: callback which is called when the message publish attempt is complete.
 
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.PipelineNotRunning` if the
             pipeline has previously been shut down
@@ -310,11 +327,16 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+
+        The following exceptions can be returned via the "error" parameter only if auto-connect
+        is enabled in the pipeline configuration:
+
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionFailedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
         logger.debug("Starting SendD2CMessageOperation on the pipeline")
@@ -331,7 +353,7 @@ class MQTTPipeline(object):
         Send an output message to the service.
 
         :param message: message to send.
-        :param callback: callback which is called when the message publish has been acknowledged by the service.
+        :param callback: callback which is called when the message publish attempt is complete.
 
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.PipelineNotRunning` if the
             pipeline has previously been shut down
@@ -339,11 +361,16 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+
+        The following exceptions can be returned via the "error" parameter only if auto-connect
+        is enabled in the pipeline configuration:
+
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionFailedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
         logger.debug("Starting SendOutputMessageOperation on the pipeline")
@@ -360,7 +387,7 @@ class MQTTPipeline(object):
         Send a method response to the service.
 
         :param method_response: the method response to send
-        :param callback: callback which is called when response has been acknowledged by the service
+        :param callback: callback which is called when response publish attempt is complete.
 
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.PipelineNotRunning` if the
             pipeline has previously been shut down
@@ -368,11 +395,16 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+
+        The following exceptions can be returned via the "error" parameter only if auto-connect
+        is enabled in the pipeline configuration:
+
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionFailedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
         logger.debug("Starting SendMethodResponseOperation on the pipeline")
@@ -390,7 +422,7 @@ class MQTTPipeline(object):
         """
         Send a request for a full twin to the service.
 
-        :param callback: callback which is called when request has been acknowledged by the service.
+        :param callback: callback which is called when request attempt is complete.
             This callback should have two parameters.  On success, this callback is called with the
             requested twin and error=None.  On failure, this callback is called with None for the
             requested win and error set to the cause of the failure.
@@ -401,11 +433,16 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+
+        The following exceptions can be returned via the "error" parameter only if auto-connect
+        is enabled in the pipeline configuration:
+
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionFailedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
         logger.debug("Starting GetTwinOperation on the pipeline")
@@ -423,7 +460,7 @@ class MQTTPipeline(object):
         Send a patch for a twin's reported properties to the service.
 
         :param patch: the reported properties patch to send
-        :param callback: callback which is called when request has been acknowledged by the service.
+        :param callback: callback which is called when the request attempt is complete.
 
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.PipelineNotRunning` if the
             pipeline has previously been shut down
@@ -431,11 +468,16 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+
+        The following exceptions can be returned via the "error" parameter only if auto-connect
+        is enabled in the pipeline configuration:
+
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionFailedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
         logger.debug("Starting PatchTwinReportedPropertiesOperation on the pipeline")
@@ -449,6 +491,7 @@ class MQTTPipeline(object):
             )
         )
 
+    # NOTE: Currently, this operation will retry itself indefinitely in the case of timeout
     def enable_feature(self, feature_name, callback):
         """
         Enable the given feature by subscribing to the appropriate topics.
@@ -463,11 +506,15 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+
+        The following exceptions can be returned via the "error" parameter only if auto-connect
+        is enabled in the pipeline configuration:
+
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionFailedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
         """
         self._verify_running()
         logger.debug("enable_feature {} called".format(feature_name))
@@ -488,6 +535,7 @@ class MQTTPipeline(object):
             )
         )
 
+    # NOTE: Currently, this operation will retry itself indefinitely in the case of timeout
     def disable_feature(self, feature_name, callback):
         """
         Disable the given feature by subscribing to the appropriate topics.
@@ -502,11 +550,15 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
+
+        The following exceptions can be returned via the "error" parameter only if auto-connect
+        is enabled in the pipeline configuration:
+
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionFailedError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ConnectionDroppedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
         :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.UnauthorizedError`
-        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.ProtocolClientError`
         """
         self._verify_running()
         logger.debug("disable_feature {} called".format(feature_name))
