@@ -10,6 +10,12 @@ Currently we have to do like this so that we don't use exec anywhere"""
 def execute_patch_for_sync():
     from azure.iot.device.iothub.sync_clients import IoTHubDeviceClient as IoTHubDeviceClient
 
+    def shutdown(self):
+        return super(IoTHubDeviceClient, self).shutdown()
+
+    shutdown.__doc__ = IoTHubDeviceClient.shutdown.__doc__
+    setattr(IoTHubDeviceClient, "shutdown", shutdown)
+
     def connect(self):
         return super(IoTHubDeviceClient, self).connect()
 
@@ -111,8 +117,27 @@ def execute_patch_for_sync():
         "create_from_x509_certificate",
         classmethod(create_from_x509_certificate),
     )
+
+    setattr(IoTHubDeviceClient, "connected", IoTHubDeviceClient.connected)
+    setattr(
+        IoTHubDeviceClient,
+        "on_method_request_received",
+        IoTHubDeviceClient.on_method_request_received,
+    )
+    setattr(
+        IoTHubDeviceClient,
+        "on_twin_desired_properties_patch_received",
+        IoTHubDeviceClient.on_twin_desired_properties_patch_received,
+    )
+
     from azure.iot.device.iothub.sync_clients import IoTHubModuleClient as IoTHubModuleClient
 
+    def shutdown(self):
+        return super(IoTHubModuleClient, self).shutdown()
+
+    shutdown.__doc__ = IoTHubModuleClient.shutdown.__doc__
+    setattr(IoTHubModuleClient, "shutdown", shutdown)
+
     def connect(self):
         return super(IoTHubModuleClient, self).connect()
 
@@ -216,6 +241,19 @@ def execute_patch_for_sync():
         "create_from_x509_certificate",
         classmethod(create_from_x509_certificate),
     )
+
+    setattr(IoTHubModuleClient, "connected", IoTHubModuleClient.connected)
+    setattr(
+        IoTHubModuleClient,
+        "on_method_request_received",
+        IoTHubModuleClient.on_method_request_received,
+    )
+    setattr(
+        IoTHubModuleClient,
+        "on_twin_desired_properties_patch_received",
+        IoTHubModuleClient.on_twin_desired_properties_patch_received,
+    )
+
     from azure.iot.device.provisioning.provisioning_device_client import (
         ProvisioningDeviceClient as ProvisioningDeviceClient,
     )
