@@ -33,6 +33,7 @@ class MQTTPipeline(object):
         # Event Handlers - Will be set by Client after instantiation of pipeline
         self.on_connected = None
         self.on_disconnected = None
+        self.on_background_exception = None
         self.on_message_received = None
         self._registration_id = pipeline_configuration.registration_id
 
@@ -114,6 +115,10 @@ class MQTTPipeline(object):
             if self.on_disconnected:
                 self.on_disconnected("disconnected")
 
+        def _on_background_exception():
+            if self.on_background_exception:
+                self.on_background_exception
+
         self._pipeline.on_pipeline_event_handler = _on_pipeline_event
         self._pipeline.on_connected_handler = _on_connected
         self._pipeline.on_disconnected_handler = _on_disconnected
@@ -180,6 +185,7 @@ class MQTTPipeline(object):
         :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.ConnectionDroppedError`
         :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.UnauthorizedError`
         :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
         logger.debug("connect called")
@@ -211,6 +217,7 @@ class MQTTPipeline(object):
 
         self._pipeline.run_op(pipeline_ops_base.DisconnectOperation(callback=pipeline_callback))
 
+    # NOTE: Currently, this operation will retry itself indefinitely in the case of timeout
     def enable_responses(self, callback=None):
         """
         Enable response from the DPS service by subscribing to the appropriate topics.
@@ -246,10 +253,16 @@ class MQTTPipeline(object):
         The following exceptions are not "raised", but rather returned via the "error" parameter
         when invoking "callback":
 
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.NoConnectionError`
+        :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.ProtocolClientError
+
+        The following exceptions can be returned via the "error" parameter only if auto-connect
+        is enabled in the pipeline configuration:
+
         :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.ConnectionFailedError`
         :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.ConnectionDroppedError`
-        :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.UnauthorizedError`
-        :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.ProtocolClientError`
+        :raises: :class:`azure.iot.device.provisioning.pipeline.exceptions.UnauthorizedError``
+        :raises: :class:`azure.iot.device.iothub.pipeline.exceptions.OperationTimeout`
         """
         self._verify_running()
 
